<?php

use Slim\{
    Factory\ServerRequestCreatorFactory,
    Views\PhpRenderer,
    Csrf\Guard,
    Psr7\Cookies
};
use DI\{
    Bridge\Slim\Bridge as SlimAppFactory,
    Container
};
use Symfony\Component\Translation\{
    Translator,
    Loader\PhpFileLoader
};
use Monolog\{
    Logger,
    Handler\StreamHandler,
    Processor\WebProcessor,
    Processor\HostnameProcessor
};
use Tuupola\Middleware\JwtAuthentication;
use App\response\customResponse;

if (empty($usercontainer=@include base_path('/angiv/config/container.php'))) {$usercontainer=[];}
return array_replace([
    'app' => function (Container $container) {
        return SlimAppFactory::create($container);
    },

    'uri' => function () {
        //RouteContext::fromRequest($request)->getRoute();
        $request=ServerRequestCreatorFactory::create()->createServerRequestFromGlobals();
        return ['path' => $request->getUri()->getPath(), 'method' => strtoupper($request->getMethod())];
    },

    'auth' => function (Container $container) {
        return new JwtAuthentication([
            'cookie' => 'accesstoken',
            'secret' => env('APP_AUTH_SECRET', ''),
            'ignore' => ['/401', '/403', '/404', '/auth' . $_ENV['APP_AUTH_URL'], '/auth/login'],
            'attribute' => 'jwtdata',
            'logger' => $container->get('logger'),
            'after' => function ($response, $arguments) {
                if (empty($_ENV['APP_LOG_USE']) || envbool('APP_LOG_USE') === true) {
                    if (envbool('APP_LOG_AUTH') === true) {
                        $logger=$GLOBALS['app']->getContainer()->get('logger');
                        $logger->error('Authorization success',  array_merge((array)$arguments['decoded']['data'], 
                            [
                                'iat' => date('Y-m-d H:i:s', $arguments['decoded']['iat']),
                                'exp' => date('Y-m-d H:i:s', $arguments['decoded']['exp'])
                            ]
                        ));
                    }
                }
            },
            'error' => function ($response, $arguments) {
                //$request=ServerRequestCreatorFactory::create()->createServerRequestFromGlobals();
                //if (!empty($refreshtoken=$request->getCookieParams()['refreshtoken'])) {}

                if (empty($_ENV['APP_LOG_USE']) || envbool('APP_LOG_USE') === true) {
                    if (envbool('APP_LOG_AUTH') === true) {
                        $logger=$GLOBALS['app']->getContainer()->get('logger');
                        $logger->error('Authorization error', (array)$arguments);
                    }
                }

                /*
                if (envbool('APP_DEBUG_USE') === true) {
                    $arguments['message']=__($arguments['message']);
                    //$scheme=parse_url($arguments['uri'], PHP_URL_SCHEME);
                    //$host=parse_url($arguments['uri'], PHP_URL_HOST);
                    //if (!empty($scheme) && !empty($host)) {$arguments['auth']=$scheme . '://' . $host . '/auth' . $_ENV['APP_AUTH_URL'];}
                    return customResponse::json($response, $arguments, 401);
                } else {
                    return $response->withHeader('Location', '/401')->withStatus(307);
                }
                */

                return $response->withHeader('Location', '/401')->withStatus(307);
            }
        ]);
    },

    'csrf' => function (Container $container) {
        return new Guard($container->get('app')->getResponseFactory());
    },

    'logger' => function () {
        $logger=new Logger(env('APP_LOG_NAME', 'Angiv'));
        $pushHandler=new StreamHandler(base_path('/angiv/tmp/log/app.log'), \Monolog\Logger::INFO);
        if (empty($_ENV['APP_LOG_HTML']) || envbool('APP_LOG_HTML') === true) {$pushHandler->setFormatter(new Monolog\Formatter\HtmlFormatter());}
        $logger->pushHandler($pushHandler);

        $logger->pushProcessor(new WebProcessor());
        $logger->pushProcessor(new HostnameProcessor());
        return $logger;
    },

    'translator' => function (Container $container) {
        $locale='';
        $currenturl=$container->get('uri')['path'];
        if (str_starts_with($currenturl, $_ENV['APP_ADMIN_URL']) !== false) {
            $locale=env('APP_LOCALE_ADMIN', '');
        } else {
            $session=new \RKA\Session(); $locale=$session->get('lang', '');
            if ($locale == '') {$locale=env('APP_LOCALE', '');}
        }
        if ($locale == 'auto' || $locale == '') {
            $locale=extension_loaded('intl') ? locale_accept_from_http($_SERVER['HTTP_ACCEPT_LANGUAGE']) : 'ru_RU';
        }
        switch (strtolower($locale)) {
            case 'en': case 'en_us': $locale='en_US'; break;
            case 'ru': case 'ru_ru': $locale='ru_RU'; break;
            default: $locale='ru_RU';
        }
        $translator=new Translator($locale);
        $translator->addLoader('php', new PhpFileLoader());
        $filelang=base_path("/angiv/asset/language/{$locale}/server.php");
        if (is_file($filelang) && is_readable($filelang)) {$translator->addResource('php', $filelang, $locale);}
        if ($_ENV['APP_CMFRUN'] == 'installer') {
            $filelang=base_path("/app/installer/asset/language/{$locale}/server.php");
            if (is_file($filelang) && is_readable($filelang)) {$translator->addResource('php', $filelang, $locale);}
        }
        return $translator;
    },

    'collection' => function (Container $container) {
        $plugins=[]; $homeurl=''; $currenturl=''; $cmfrunpart='user';
        $currenturl=$container->get('uri')['path'];
        if (str_starts_with($currenturl, $_ENV['APP_ADMIN_URL']) !== false) {$cmfrunpart='admin';}
        $pluginsjsonpath=base_path('/angiv/plugin/plugins.json');
        if (is_file($pluginsjsonpath) && is_readable($pluginsjsonpath)) {
            $json=file_get_contents($pluginsjsonpath);
            if (!empty($json) && $json != 'null') {
                $plugins=json_decode($json, true);
                if (json_last_error() != JSON_ERROR_NONE) {
                    $plugins=json_last_error_msg();
                } else {
                    if (!empty($plugins) && count($plugins) != 0) {
                        usort($plugins, function ($item1, $item2) {
                            return ($item1['priority'] <= $item2['priority']) ? -1 : 1;
                        });
                        foreach ($plugins as $key => $plugin) {
                            if (!empty($plugin['homeurl'])) {
                                $homeurl=$plugin['homeurl'];
                                if ($cmfrunpart == 'user' && $plugin['isused'] != 'on') {$homeurl='';}
                            }

                            $plugins[$key]['isremoved']='no';
                            $plugins[$key]['pagesusercount']=0;
                            $plugins[$key]['pagesadmincount']=0;
                            $plugins[$key]['pagesuser']=[];
                            $plugins[$key]['pagesadmin']=[];
                            $indexphppath=base_path('/angiv/plugin/' . $plugin['dir'] . '/index.php');
                            if (is_file($indexphppath) && is_readable($indexphppath)) {
                                $plugininfo=@include $indexphppath;
                                if ($currenturl != $_ENV['APP_ADMIN_URL'] . '/plugins') {
                                    $plugins[$key]['css' . $cmfrunpart]=$plugininfo['css' . $cmfrunpart] ?? [];

                                    $plugins[$key]['js' . $cmfrunpart]=$plugininfo['js' . $cmfrunpart] ?? [];

                                    $plugins[$key]['ajax' . $cmfrunpart]=$plugininfo['ajax' . $cmfrunpart] ?? '';

                                    if (!empty($plugininfo['pages' . $cmfrunpart]) && count($plugininfo['pages' . $cmfrunpart]) != 0) {
                                        foreach ($plugininfo['pages' . $cmfrunpart] as $page) {
                                            if (!empty($page['url']) && !empty($page['controller'])) {
                                                $page['title']=(!empty($page['title'])) ? __($page['title']) : $page['url'];
                                                if ($cmfrunpart == 'user') {
                                                    $page['url']=str_replace('//', '/', '/' . $page['url']);
                                                } else {
                                                    $page['url']='/' . $plugin['dir'] . str_replace('//', '/', '/' . $page['url']);
                                                }
                                                $controller=str_replace('//', '/', $plugin['dir'] . '/' . str_replace('.php', '', $page['controller']));
                                                $controller=str_replace('/', '\\', 'Plugin/' . $controller);
                                                $page['controller']=$controller;
                                                $plugins[$key]['pages' . $cmfrunpart][]=$page;
                                            }
                                        }
                                    }
                                }
                                $plugins[$key]['pagesusercount']=count($plugininfo['pagesuser'] ?? []);
                                $plugins[$key]['pagesadmincount']=count($plugininfo['pagesadmin'] ?? []);
                            } else {$plugins[$key]['isremoved']='yes';}
                        }
                    }
                }
            }
        }
        return ['plugins' => $plugins, 'homeurl' => $homeurl];
    }
], $usercontainer);