<?php

if (!function_exists('base_path')) {
    function base_path(string $path = ''): string {
        return $_SERVER['DOCUMENT_ROOT'] . $path;
    }
}

if (!function_exists('html')) {
    function html(string $text = null): string {
        return htmlspecialchars($text, ENT_QUOTES | ENT_SUBSTITUTE | ENT_HTML401, 'UTF-8');
    }
}

if (!function_exists('evalbool')) {
    function evalbool(string $value = null): bool {
        return (!empty($value) ? (strcasecmp($value, 'true') ? false : true) : false);
    }
}

if (!function_exists('env')) {
    function env(string $name = '', string $default = null): string {
        $value=$_ENV[$name] ?? null;
        if (!empty($value) || $value == '0') {return html($value);}
        if (!empty($default) || $default === '') {return $default;}
        throw new \Exception(__('Undefined env variable') . ' ' . $name, 400);
    }
}

if (!function_exists('envbool')) {
    function envbool(string $name = ''): bool {
        $value=$_ENV[$name] ?? null;
        return (!empty($value) ? (strcasecmp($value, 'true') == 0 ? true : false) : false);
    }
}

if (!function_exists('__')) {
    function __(string $msg = '', string $locale = ''): string {
        $translator=$GLOBALS['app']->getContainer()->get('translator');
        return html($translator->trans($msg, [], 'messages', (($locale != '') ? $locale : $translator->getLocale())));
    }
}

if (!function_exists('routelist')) {
    function routelist(string $class = ''): array {
        $res=[];
        if (class_exists($class)) {
            $ref=new ReflectionClass($class);
            $methods=$ref->getMethods(\ReflectionMethod::IS_PUBLIC);
            foreach ($methods as $method) {
                if (!$method->isConstructor()) {
                    $comments=trim($method->getDocComment());
                    if (preg_match("/route\((.+)\)/i", $comments, $item) == 1) {
                        $route=json_decode($item[1], true);
                        $route['methods']=array_flip(array_change_key_case(array_flip($route['methods']), CASE_UPPER));
                        $route['action']=$class . '::' . $method->getName();
                        if (empty($route['csrf'])) {$route['csrf']=false;}
                        $res[]=$route;
                    }
                }
            }
            unset($ref); unset($methods);
        }
        return $res;
    }
}

if (!function_exists('filelist')) {
    function filelist(array $list = null, string $dir = null): array {
        $res=[];
        if (!empty($list) && count($list) != 0) {
            foreach ($list as $item) {
                $filepath='';
                $filename=(!empty($item['href']) ? $item['href'] : (!empty($item['src']) ? $item['src'] : $item));
                if (str_starts_with($filename, '/custom/') !== false) {$filepath='/angiv/asset/lib/' . str_replace('/custom/', '', $filename);}
                if (str_starts_with($filename, '/system/') !== false) {$filepath='/app/asset/lib/' . str_replace('/system/', '', $filename);}
                if (empty($filepath) && $filename[0] == '/') {$filepath=$filename;}
                if (empty($filepath) && !empty($dir)) {$filepath="/angiv/plugin/{$dir}/{$filename}";}
                if (!empty($filepath)) {
                    if (is_file(base_path($filepath)) && is_readable(base_path($filepath))) {
                        $ext=strtolower(pathinfo($filepath, PATHINFO_EXTENSION));
                        switch ($ext) {
                            // -----css
                            case 'css':
                                $res[]=[
                                    'href' => $filepath,
                                    'attr' => (!empty($item['attr']) ? $item['attr'] : '')
                                ];
                                break;
                            // -----js
                            case 'js':
                                $res[]=[
                                    'type' => (!empty($item['type']) ? $item['type'] : 'text/javascript'),
                                    'src' => $filepath,
                                    'attr' => (!empty($item['attr']) ? (' ' . $item['attr']) : '')
                                ];
                                break;
                            // -----php
                            case 'php':
                                $res[]=base_path($filepath);
                                break;
                        }
                    }
                }
            }
        }
        return $res;
    }
}