<?php

namespace App\middleware;

use Slim\{
    Psr7\Request,
    Psr7\Response
};
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;

class ipaccessMiddleware
{
    protected $settings;

    public function __construct(array $settings = []) {
        $this->settings=array_change_key_case($settings, CASE_LOWER);
    }

    public function __invoke(Request $request, RequestHandler $handler) {
        $access=0;
        if (isset($this->settings['all'])) {$access=$this->settings['all'];} else {
            if (!empty($currentip=$request->getAttribute('ipaddress'))) {
                foreach ($this->settings as $ip => $allow) {
                    if ($this->ipmatch($ip, $currentip)) {$access=$allow; break;}
                }
            }
        }

        if ($access === 0) {
            $response=new Response();
            return $response->withHeader('Location', '/403')->withStatus(307);
        } else {
            return $handler->handle($request);
        }
    }

    private function ipmatch($ip, $currentip) {
        list($subnet, $slash, $size)=preg_split('@(/|$)@', $ip, 2, PREG_SPLIT_DELIM_CAPTURE);
        if (filter_var($subnet, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
            $size=32-intval($slash ? $size : 32);
            return (ip2long($subnet) == (ip2long($currentip) & (-1 << $size)));
        } elseif (filter_var($subnet, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
            $size=intval($slash ? $size : 128);
            return (inet_pton($subnet) == (inet_pton($currentip) & pack('H*', str_pad(str_repeat('f', $size / 4) . ['', '8', 'c', 'e'][$size % 4], 32, '0'))));
        }
        return false;
    }
}