<?php

namespace App\middleware;

use Slim\Psr7\{
    Request,
    Response
};
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use DI\Container;
use Firebase\JWT\{
    JWT,
    Key
};

class rendererMiddleware
{
    protected $container;
    protected $view;
    protected $plugin;
    protected $page;

    public function __construct(
        Container $container,
        $view = null,
        $plugin = null,
        $page = null
    )
    {
        $this->container=$container;
        $this->view=$view;
        $this->plugin=$plugin;
        $this->page=$page;
    }

    public function __invoke(Request $request, RequestHandler $handler) {
        if (!empty($this->view) && $this->container->has($this->view)) {
            $renderer=$this->container->get($this->view);

            if (!empty($this->page['title'])) {
                $renderer->addAttribute('title', $this->page['title']);
            }

            if (!empty($this->page['seo'])) {
                $renderer->addAttribute('seo', 
                    array_replace([
                        'description' => '',
                        'keywords' => '',
                        'canonicalurl' => ''
                    ], $this->page['seo'])
                );
            }

            if (!empty($this->page['ogseo'])) {
                $renderer->addAttribute('ogseo', 
                    array_replace([
                        'ogtitle' => '',
                        'ogsitename' => '',
                        'ogdescription' => '',
                        'ogtype' => '',
                        'ogurl' => '',
                        'ogimage' => ''
                    ], $this->page['ogseo'])
                );
            }

            if (!empty($this->plugin) && !empty($this->page['page'])) {$renderer->addAttribute('currentpage', "{$this->plugin['dir']}/{$this->page['page']}");}

            if (!empty($this->plugin)) {
                $renderer->addAttribute('css', array_merge(
                    filelist($this->plugin[($this->view == 'viewadmin') ? 'cssadmin' : 'cssuser'], $this->plugin['dir']),
                    (!empty($this->page['css']) ? filelist($this->page['css'], $this->plugin['dir']) : [])
                ));
            }

            if (!empty($this->plugin)) {
                $renderer->addAttribute('js', array_merge(
                    filelist($this->plugin[($this->view == 'viewadmin') ? 'jsadmin' : 'jsuser'], $this->plugin['dir']),
                    (!empty($this->page['js']) ? filelist($this->page['js'], $this->plugin['dir']) : [])
                ));
            }

            $phpcsrf=(!empty($request) ? $request->getAttribute('csrf_name', '') . '|' . $request->getAttribute('csrf_value', '') : '|');
            $renderer->addAttribute('phpcsrf', '"' . (($phpcsrf == '|') ? '' : $phpcsrf) . '"');

            $phpjwtdata=null;
            if (!empty($request->getAttribute('jwtdata'))) {
                $phpjwtdata=$request->getAttribute('jwtdata');
            } else {
                if (isset($_COOKIE['accesstoken'])) {
                    $phpjwtdata=JWT::decode($_COOKIE['accesstoken'], new Key(env('APP_AUTH_SECRET', ''), 'HS256'));
                }
            }
            $renderer->addAttribute('phpjwtdata', $phpjwtdata);
        }

        return $handler->handle($request);
    }
}