<?php

namespace App\response;

use Slim\Interfaces\ErrorRendererInterface;
use Throwable;

class errorResponse implements ErrorRendererInterface
{
    public function __invoke(Throwable $exception, bool $displayErrorDetails): string {
        if ($displayErrorDetails) {
            $code=$exception->getCode();
            $html=sprintf('<div><b>' . __('Type') . ':</b> %s</div>', get_class($exception));
            $html.=sprintf('<div><b>' . __('Code') . ':</b> %s</div>', $code);
            $html.=sprintf('<div><b>' . __('Message') . ':</b> %s</div>', htmlentities($exception->getMessage()));
            $html.=sprintf('<div><b>' . __('File') . ':</b> %s</div>', $exception->getFile());
            $html.=sprintf('<div><b>' . __('Line') . ':</b> %s</div>', $exception->getLine());
            $html.='<h2>' . __('Description') . '</h2>';
            $html.=sprintf('<pre>%s</pre>', htmlentities($exception->getTraceAsString()));
        } else {
            $html="<p>{$this->getErrorDescription($exception)}</p>";
        }

        return sprintf(
            '<!doctype html>' .
            '<html lang="en">' .
            '    <head>' .
            '        <meta charset="utf-8">' .
            '        <meta name="viewport" content="width=device-width, initial-scale=1">' .
            '        <title>Application Error</title>' .
            '        <style>' .
            '            body {margin: 0; padding: 30px; font: 12px/1.5 Helvetica, Arial, Verdana, sans-serif;}' .
            '            h1 {margin: 0px 0px 30px 0px; font-size: 32px; font-weight: normal; line-height: 32px;}' .
            '        </style>' .
            '    </head>' .
            '    <body>' .
            '        <h1>' . __('Application Error') . '</h1>' .
            '        <div>%s</div>' .
            '    </body>' .
            '</html>',
            $html
        );
    }
}